/*
 * 	MacRoute - User Space interface to Appletalk-IP decapsulation.
 *      Written By: Jay Schulist <Jay.Schulist@spacs.k12.wi.us>
 *                  Copyright (C) 1997 Jay Schulist
 *
 * This software may be used and distributed according to the terms
 * of the GNU Public License, incorporated herein by reference.
 */

#include "MacRoute.h"

char *in_ntoa(__u32 in)
{
        static char buff[18];
        char *p;

        p = (char *) &in;
        sprintf(buff, "%d.%d.%d.%d",
                (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));
        return(buff);
}

int main (int argc, char *argv[])
{
	struct ipddp_route rt;
	__u32 ip;
	struct at_addr at;
	struct ifreq ifr;
	int SockConfig, c;
	char command[25];	/* Large for silly users, untill we get some checks in here. */

	while((c=getopt(argc, argv, "cnmi")) != EOF)
	{
        	switch(c)
                {
			/* Get the command use wants us to do  add/del */
			 case 'c':
				strncpy(command, argv[optind++], 4);
				break;

			/* Get the Mac's network number */
			case 'n':
				at.s_net = atoi(argv[optind++]);
				break;

			/* Get the Mac's node number */
			case 'm':
				at.s_node = atoi(argv[optind++]);
				break;

			/* Get the Mac's IP number */
			case 'i':
				ip = inet_addr(argv[optind++]);
				break;

			default:
				printf("MacRoute Options:\n");
                		printf("        -c      add/del this route\n");
                		printf("        -n      Mac's Network number\n");
                		printf("        -m      Mac's Node number\n");
                		printf("        -i      Mac's IP number\n");
                		printf("Example: MacRoute -c add -n 1000 -m 7 -i 192.168.1.5\n");
                		exit(1);
		}
	}

	SockConfig = socket(AF_INET,SOCK_DGRAM,0);

	/* Put our information into ifr */
	strcpy(ifr.ifr_name,"ipddp0");
        ifr.ifr_data = (void *)&rt;
        rt.ip = ip;
        rt.at.s_net = htons(at.s_net);
	rt.at.s_node = at.s_node;

	if(strcmp("add", command) == 0)
	{
       		if(ioctl(SockConfig,SIOCADDIPDDPRT,&ifr)<0) 
		{
			perror("SIOCADDIPDDPRT");
			exit(1);
		}
		printf("MacRoute: Added route Net:Node %d:%d to IP %s\n", ntohs(rt.at.s_net), 
			rt.at.s_node, in_ntoa(rt.ip));
		exit(0);
	}

	/*
	 * This will require Net:Node and IP, once kernel supports it.
	 */
	if(strcmp("del", command) == 0)
	{
		if(ioctl(SockConfig,SIOCDELIPDDPRT,&ifr)<0)
                	perror("SIOCDELIPDDPRT");
		exit(0);
	}

	return 1;
}
