/*
 * MacReg - Utility to register your machine as an IPGATEWAY
 * Written By: Jay Schulist <Jay.Schulist@spacs.k12.wi.us>
 *                  Copyright (C) 1997 Jay Schulist
 *
 * This software may be used and distributed according to the terms
 * of the GNU Public License, incorporated herein by reference.
 */

#include "MacRoute.h"

void DelGateway(__u32 ipaddr);
void AddGateway(__u32 ipaddr);

char *in_ntoa(__u32 in)
{
        static char buff[18];
        char *p;

        p = (char *) &in;
        sprintf(buff, "%d.%d.%d.%d",
                (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));
        return(buff);
}

/*
 * Get a device's IP number for IPDDP
 */
__u32 GetIP(char *device)
{
        struct sockaddr_in *s_in;
        struct ifreq ifr;
        int SockFD;

        SockFD = socket(AF_INET, SOCK_DGRAM, 0);

        strcpy(ifr.ifr_name, device);
        if (ioctl(SockFD, SIOCGIFADDR, &ifr) < 0)
                perror("SIOCGIFADDR");
        s_in = (struct sockaddr_in *)&ifr.ifr_addr;

        return(s_in->sin_addr.s_addr);
}

void RegIPGate(__u32 ipaddr);
void RegIPaddr(__u32 ipaddr);

ATP atp;
int SockLTalk;  /* Socket descriptor. */


void main (void)
{
	__u32 IPGateIPDDP;
        __u32 IPGateETH;
	
	IPGateIPDDP = GetIP(IPDDP_DEV);
	IPGateETH = GetIP(ETH_DEV);

	printf("%s\n", in_ntoa(IPGateIPDDP));
	printf("%s\n", in_ntoa(IPGateETH));

	atp = atp_open(72);  /* IP DDP Port (72). */
        if(atp == NULL)
                perror("atp_open()");

	/* Socket into kernel Appletalk layer. */
        SockLTalk = socket(AF_APPLETALK, SOCK_DGRAM, 0);
        if(SockLTalk < 0)
                perror("socket()");

    	RegIPGate(IPGateETH);
	RegIPaddr(IPGateIPDDP);
    	RegIPaddr(IPGateETH);

    	printf("Registered our IPs\n");

	return;
}

/*
 * Register our Ethernet IP Gateway as the IPGATEWAY and IPADDRESS.
 */
void RegIPGate(__u32 ipaddr)
{
        if (nbp_rgstr(atp_sockaddr(atp),in_ntoa(ipaddr),"IPGATEWAY","*")<0)
                printf("IPGATEWAY nbp_rgstr failed.\n");
        else
                printf("Registered %s as IPGATEWAY\n", in_ntoa(ipaddr));

        return;
}

void RegIPaddr(__u32 ipaddr)
{
        if(nbp_rgstr(atp_sockaddr(atp),in_ntoa(ipaddr),"IPADDRESS","*")<0)
                printf("IPADDRESS nbp_rgstr failed.\n");
        else
                printf("Registered %s as IPADDRESS\n", in_ntoa(ipaddr));

        return;
}
